/*
 * BlinkMMulti -- 
 *
 *
 * BlinkM connections to Arduino
 * PWR - -- gnd -- black -- Gnd
 * PWR + -- +5V -- red   -- 5V
 * I2C d -- SDA -- green -- Analog In 4
 * I2C c -- SCK -- blue  -- Analog In 5
 *
 *
 * 2007, Tod E. Kurt, ThingM, http://thingm.com/
 *
 */


#include "Wire.h"
#include "BlinkM_funcs.h"  // include in same folder as .ino

byte i;
byte Dir;
byte cmd;
byte Red;
byte Blue;
byte Green;
byte PixelAddress = 0;
volatile byte IncomingByte = 0;
byte I2CFlag = 0;
unsigned long MainTimestamp; // used to switch between static lighting sequences
unsigned long Timestamp;  // used to time moodlighting LED changes
int DelayTime;

void setup()
{
  randomSeed(analogRead(0));
  Wire.onReceive(Intr);
  Wire.begin(42);
}

void loop() {
 if (I2CFlag > 0) {  // checks for I2C flag and resets if triggered
    I2CFlag = 0;
  }
  else {
    while (I2CFlag == 0) {  // infinite loop waiting for flag if it's not
    }
  }
  
//Notes:
// Modes: 0= ack(flash), 1=off, 2=walking, 3=fanmode, 4&5=avoidance, 6=static
// 2: larson
// 3: seahawkcelebrate, flash, seahawkpulse
// 4&5: blinky, redswell, directionalscanner
// 6: moodlighting, moodswell, moodindividual

  switch (IncomingByte) {
    case 0:  // ack
      BlinkM_off(0);
      DelayTime = 2500;
      Flash();
      break;
    case 1:  // off
      BlinkM_off(0);
      break;
    case 2:  // walking
      BlinkM_off(0);
      do {
        Larson();
      }
      while (I2CFlag == 0);
      break;
    case 3:  // fanmode  
      BlinkM_off(0);
      i = random(3);  // random selection of lighting pattern
      switch(i) {
        case 0:
          SeahawkParade();
          break;
        case 1:
          BlinkM_setFadeSpeed(0, 2);
          DelayTime = 1000;
          do {  // do-while needed as Flash() runs only once usually
            Flash();
          }
          while (I2CFlag == 0);
          break;
        case 2:
          SeahawkPulse();
          break;
      }
      break;
    case 4: // avoidance sequence with right directional byte 
      BlinkM_off(0);
      Avoidance();
      break;
    case 5:  // avoidance with left directional
      BlinkM_off(0);
      Avoidance();
      break;
    case 6:  // static
      int j;  // used to adjust maintimestamp
      BlinkM_off(0);
      do {  // I2C flag watcher
        j = random(10000,20000);  // time between sequence changes
        MainTimestamp = millis() + j;
        i = random(2);
        switch(i) {
          case 0:
            do {
              MoodLighting();
            }
            while(millis() < MainTimestamp);
            break;
          case 1:
            do {
              MoodSwell();
            }
            while(millis() < MainTimestamp);
            break;
          case 2:
            do {
              MoodIndividual();
            }
            while(millis() < MainTimestamp);
            break;
        }
      }
      while(I2CFlag == 0); 
      break; 
  }
}


void Flash() {
  BlinkM_setRGB(0, 255, 255, 255);
  BlinkM_fadeToRGB(0, 0, 0, 0);
  delay(DelayTime);
}
void Larson() {
  BlinkM_setRGB(1, 255, 0, 0);
  delay(100);
  BlinkM_fadeToRGB(2, 0, 0, 0);
  delay(600);
  BlinkM_setRGB(2, 255, 0, 0);
  delay(100);
  BlinkM_fadeToRGB(1, 0, 0, 0);
  delay(1100);
  BlinkM_setRGB(3, 255, 0, 0);
  delay(100);
  BlinkM_fadeToRGB(2, 0, 0, 0);
  delay(1100);
  BlinkM_setRGB(2, 255, 0, 0);
  delay(100);
  BlinkM_fadeToRGB(3, 0, 0, 0);
  delay(1100);
  BlinkM_setRGB(1, 255, 0, 0);
  delay(100);
  BlinkM_fadeToRGB(2, 0, 0, 0);
  delay(300);
}
void SeahawkParade() {
  do {
    for (i=1; i<4; i++) {
      switch (i) {
        case 1:
          BlinkM_setRGB(i, 0, 0, 255);
          BlinkM_setRGB(2, 0, 255, 0);
          BlinkM_setRGB(3, 0, 255, 0);
          delay(100);
          break;
        case 2:
          BlinkM_setRGB(i, 0, 0, 255);
          BlinkM_setRGB(1, 0, 255, 0);
          BlinkM_setRGB(3, 0, 255, 0);
          delay(100);
          break;
        case 3:
          BlinkM_setRGB(i, 0, 0, 255);
          BlinkM_setRGB(1, 0, 255, 0);
          BlinkM_setRGB(2, 0, 255, 0);
          delay(100);
          break;
      }
    }
  }
  while(I2CFlag = 0);
}
void SeahawkPulse() {
  do {
    for(i=0; i<2; i++);
      BlinkM_setFadeSpeed(0, 3);  // fast fade
      if(i == 0) {
        BlinkM_fadeToRGB(0, 0, 0, 255);  // to blue
      }
      else {
        BlinkM_fadeToRGB(0, 0, 255, 0);  // to green
      }
    delay(500);
    BlinkM_setFadeSpeed(0, 2);  // slower fade
    BlinkM_fadeToRGB(0, 0, 0, 0);  // to black
    delay(1000);
  }
  while(I2CFlag == 0);
}
void Avoidance() {    
  i = random(3);  // sequence selection
  switch(i) {
    case 0: // blinky
      Blinky();
      break;
    case 1:  // redswell
      RedSwell();
      break;
    case 2: // directional
      switch(IncomingByte) {
        case 4:  // pattern moves right
          do {
            BlinkM_setRGB(1, 0, 0, 0);
            BlinkM_setRGB(2, 0, 0, 0);
            BlinkM_setRGB(3, 255, 0, 0);
            delay(50);
            BlinkM_setRGB(1, 0, 0, 0);
            BlinkM_setRGB(2, 255, 0, 0);
            BlinkM_setRGB(3, 0, 0, 0);
            delay(50);
            BlinkM_setRGB(1, 255, 0, 0);
            BlinkM_setRGB(2, 0, 0, 0);
            BlinkM_setRGB(2, 0, 0, 0);
            delay(50);
          }
          while(I2CFlag == 0);
          break;
    case 5:  // pattern moves left
      do {
        BlinkM_setRGB(1, 255, 0, 0);
        BlinkM_setRGB(2, 0, 0, 0);
        BlinkM_setRGB(2, 0, 0, 0);
        delay(100);
        BlinkM_setRGB(1, 0, 0, 0);
        BlinkM_setRGB(2, 255, 0, 0);
        BlinkM_setRGB(3, 0, 0, 0);
        delay(100);
        BlinkM_setRGB(1, 0, 0, 0);
        BlinkM_setRGB(2, 0, 0, 0);
        BlinkM_setRGB(3, 255, 0, 0);
        delay(100);
      }
      while(I2CFlag == 0);
      break;
    }
  }
}
void Blinky() {
  int Color;  // will hold value selecting red, green, blue, yellow, purple, or cyan
  do {
    PixelAddress = random(3) + 1;
    Color = random(5);
    switch (Color) {
      case 0:  // red
        BlinkM_setRGB(PixelAddress, 255, 0, 0);
        break;
      case 1:  // green
        BlinkM_setRGB(PixelAddress, 0, 255, 0);
        break;
      case 2:  // blue
        BlinkM_setRGB(PixelAddress, 0, 0, 255);
        break;
      case 3:  // yellow
        BlinkM_setRGB(PixelAddress, 255, 255, 0);
        break;
      case 4:  // purple
        BlinkM_setRGB(PixelAddress, 255, 0, 255);
        break;
      case 5:  // cyan
        BlinkM_setRGB(PixelAddress, 0, 255, 255);
        break;
    }
    delay(250);
  }
  while(I2CFlag == 0);  
}
void RedSwell() {
  BlinkM_setFadeSpeed(0, 2);  // fastish fade
  do {
    for(i=0; i<2; i++);
      if(i == 0) {
        BlinkM_fadeToRGB(0, 255, 0, 0);
      }
      else {
        BlinkM_fadeToRGB(0, 0, 0, 0);
      }
    delay(1000);
  }
  while(I2CFlag == 0);
}
void MoodLighting() {
  int FadeSpeed;
  while(millis() < Timestamp) {
  }
  PixelAddress = random(3) + 1;
  Red = random(256);
  Green = random(256);
  Blue = random(256);
  FadeSpeed = random(1,3);
  BlinkM_setFadeSpeed(PixelAddress, FadeSpeed);
  BlinkM_fadeToRGB(PixelAddress, Red, Green, Blue);
  Timestamp = random(500,2000) + millis();
}
void MoodSwell() {
  int FadeSpeed;
  Red = random(256);
  Green = random(256);
  Blue = random(256);
  FadeSpeed = random(1,3);
  BlinkM_setFadeSpeed(0, FadeSpeed);
  BlinkM_fadeToRGB(0, Red, Green, Blue);
  // add delay
  FadeSpeed = random(1,3);
  BlinkM_setFadeSpeed(0, FadeSpeed);
  BlinkM_fadeToRGB(0, 0, 0, 0);
  // add delay
}
void MoodIndividual() {
  int FadeSpeed;
  PixelAddress = random(3) + 1;
  Red = random(256);
  Green = random(256);
  Blue = random(256);
  FadeSpeed = random(1,3);
  BlinkM_setFadeSpeed(PixelAddress, FadeSpeed);
  BlinkM_fadeToRGB(PixelAddress, Red, Green, Blue);
  // add delay
  FadeSpeed = random(1,3);
  BlinkM_setFadeSpeed(PixelAddress, FadeSpeed);
  BlinkM_fadeToRGB(PixelAddress, 0, 0, 0);
  // add delay
}
void Intr(int byteCount) {
  I2CFlag = 1;
  IncomingByte = Wire.read();
}

